@@ -1,10 +1,9 @@
 import logging
-from os import getcwd
 from os.path import dirname
 
-import pkg_resources
+from pkg_resources import get_distribution
 
-__version__ = pkg_resources.get_distribution('sonusai').version
+__version__ = get_distribution('sonusai').version
 BASEDIR = dirname(__file__)
 
 # create logger
@@ -42,11 +41,16 @@ def update_console_handler(verbose: bool):
 def initial_log_messages(name: str):
     from datetime import datetime
     from getpass import getuser
+    from os import getcwd
     from socket import gethostname
+    from sys import argv
 
-    logger.info(f'SonusAI {name} {__version__}')
+    logger.info(f'SonusAI {__version__}')
+    logger.info(f'{name}')
+    logger.info('')
     logger.debug(f'Host:      {gethostname()}')
     logger.debug(f'User:      {getuser()}')
     logger.debug(f'Directory: {getcwd()}')
     logger.debug(f'Date:      {datetime.now()}')
+    logger.debug(f'Command:   {" ".join(argv)}')
     logger.debug('')
