@@ -1,24 +1,20 @@
 """sonusai evaluate
 
-usage: evaluate [-hv] (-f FEATURE) (-n PREDICT) [-t PTHR]
+usage: evaluate [-hv] [-i MIXID] (-f FEATURE) (-p PREDICT) [-t PTHR] LOC
 
 options:
-   -h, --help
-   -v, --verbose                    Be verbose.
-   -f FEATURE, --feature FEATURE    Feature + truth .h5 data file
-   -n PREDICT, --predict PREDICT    Predict .h5 data file.
-   -t PTHR, --thr PTHR              Optional prediction decision threshold(s). [default: 0].
+    -h, --help
+    -v, --verbose                   Be verbose.
+    -i MIXID, --mixid MIXID         Mixture ID(s) to generate. [default: *].
+    -p PREDICT, --predict PREDICT   A directory containing prediction data.
+    -t PTHR, --thr PTHR             Optional prediction decision threshold(s). [default: 0].
 
-Evaluate calculates performance metrics of neural-network models from model prediction data and the associated
-feature+truth data file.
+Evaluate calculates performance metrics of neural-network models from model prediction data and genft data.
 
 Inputs:
-    FEATURE     A SonusAI feature+truth HDF5 file. Contains:
-                    attribute:  mixdb
-                    dataset:    feature   Note: not required for metrics
-                    dataset:    truth_f
-                    dataset:    segsnr    Optional
-    PREDICT     A SonusAI predict HDF5 file. Contains:
+    LOC         A SonusAI mixture database directory.
+    MIXID       A glob of mixture ID(s) to generate.
+    PREDICT     A directory containing SonusAI predict HDF5 files. Contains:
                     dataset:    predict (either [frames, num_classes] or [frames, timesteps, num_classes])
     PTHR        Scalar or array of thresholds. Default 0 will select values:
                     argmax()    if mixdb indicates single-label mode (truth_mutex = true)
@@ -26,22 +22,26 @@ Inputs:
                 If PTHR = -1, optimal thresholds are calculated using precision_recall_curve() which
                 optimizes F1 score.
 """
-from pathlib import Path
 from typing import Union
 
 import numpy as np
 
 from sonusai import logger
+from sonusai.mixture import Feature
+from sonusai.mixture import Location
 from sonusai.mixture import MixtureDatabase
+from sonusai.mixture import Predict
+from sonusai.mixture import Segsnr
+from sonusai.mixture import Truth
 
 
 def evaluate(mixdb: MixtureDatabase,
-             truth: np.ndarray,
-             predict: Union[None, np.ndarray],
-             segsnr: Union[None, np.ndarray] = None,
-             output_dir: Union[None, str, Path] = None,
+             truth: Truth,
+             predict: Predict = None,
+             segsnr: Segsnr = None,
+             output_dir: Location = None,
              predict_thr: Union[float, np.ndarray] = 0.0,
-             feature: Union[None, np.ndarray] = None,
+             feature: Feature = None,
              verbose: bool = False) -> None:
     from os.path import join
 
@@ -52,8 +52,9 @@ def evaluate(mixdb: MixtureDatabase,
     from sonusai.metrics import snr_summary
     from sonusai.mixture import SAMPLE_RATE
     from sonusai.queries import get_mixids_from_snr
+    from sonusai.utils import get_num_classes_from_predict
     from sonusai.utils import human_readable_size
-    from sonusai.utils import reshape_truth_predict
+    from sonusai.utils import reshape_outputs
     from sonusai.utils import seconds_to_hms
 
     update_console_handler(verbose)
@@ -63,10 +64,11 @@ def evaluate(mixdb: MixtureDatabase,
         logger.exception(f'Number of classes in truth and predict are not equal. Exiting.')
         raise SystemExit(1)
 
-    # truth, predict can be either frames x num_classes, or frames x timesteps x num_classes
+    # truth, predict can be either [frames, num_classes] or [frames, timesteps, num_classes]
     # in binary case dim may not exist, detect this and set num_classes == 1
     timesteps = -1
-    truth, predict, num_classes = reshape_truth_predict(truth, predict, timesteps)
+    predict, truth = reshape_outputs(predict=predict, truth=truth, timesteps=timesteps)
+    num_classes = get_num_classes_from_predict(predict=predict, timesteps=timesteps)
 
     fdiff = truth.shape[0] - predict.shape[0]
     if fdiff > 0:
@@ -81,7 +83,7 @@ def evaluate(mixdb: MixtureDatabase,
     # Check segsnr, input is always in transform frames
     compute_segsnr = False
     if len(segsnr) > 0:
-        segsnr_feature_frames = segsnr.shape[0] / (mixdb.feature_step_samples / mixdb.frame_size)
+        segsnr_feature_frames = segsnr.shape[0] / (mixdb.feature_step_samples / mixdb.fg.ftransform_R)
         if segsnr_feature_frames == truth.shape[0]:
             compute_segsnr = True
         else:
@@ -116,7 +118,7 @@ def evaluate(mixdb: MixtureDatabase,
 
     # Summarize the mixture data
     num_mixtures = len(mixdb.mixtures)
-    total_samples = sum([sub.samples for sub in mixdb.mixtures])
+    total_samples = sum([mixture.samples for mixture in mixdb.mixtures])
     duration = total_samples / SAMPLE_RATE
 
     logger.info('')
@@ -188,21 +190,26 @@ def evaluate(mixdb: MixtureDatabase,
 def main():
     from datetime import datetime
     from os import mkdir
+    from os.path import join
 
     import h5py
     from docopt import docopt
 
     import sonusai
+    from sonusai import SonusAIError
     from sonusai import create_file_handler
-    from sonusai.mixture import mixdb_from_json
     from sonusai.utils import read_predict_data
     from sonusai.utils import trim_docstring
 
     args = docopt(trim_docstring(__doc__), version=sonusai.__version__, options_first=True)
 
+    verbose = args['--verbose']
     feature_name = args['--feature']
     predict_name = args['--predict']
     predict_threshold = float(args['--thr'])
+    location = args['LOC']
+
+    mixdb = MixtureDatabase(config=location, show_progress=True)
 
     # create output directory
     output_dir = f'evaluate-{datetime.now():%Y%m%d}'
@@ -213,20 +220,14 @@ def main():
         try:
             mkdir(output_dir)
         except OSError as error:
-            logger.error(f'Could not create directory, {output_dir}: {error}')
-            raise SystemExit(1)
+            raise SonusAIError(f'Could not create directory, {output_dir}: {error}')
 
-    log_name = output_dir + '/evaluate.log'
-    create_file_handler(log_name)
+    create_file_handler(join(output_dir, 'evaluate.log'))
 
-    # mixdb, feature, truth, segsnr = read_feature_data(feature_name)
     with h5py.File(feature_name, 'r') as f:
-        mixdb = mixdb_from_json(f.attrs['mixdb'])
-        # feature = f['/feature'][:]  # No feature load saves significant time
-        truth_f = np.array(f['/truth_f'])
-        segsnr = np.array(f['/segsnr'])
+        truth_f = np.array(f['truth_f'])
+        segsnr = np.array(f['segsnr'])
 
-    # mixture, target, noise = read_mixture_data(mixture_name)
     predict = read_predict_data(predict_name)
 
     evaluate(mixdb=mixdb,
@@ -235,7 +236,7 @@ def main():
              output_dir=output_dir,
              predict=predict,
              predict_thr=predict_threshold,
-             verbose=args['--verbose'])
+             verbose=verbose)
 
     logger.info(f'Wrote results to {output_dir}')
 
