@@ -1,12 +1,15 @@
 """sonusai genmixdb
 
-usage: genmixdb [-hv] CONFIG...
+usage: genmixdb [-hvmfs] LOC
 
 options:
    -h, --help
    -v, --verbose    Be verbose.
+   -m, --mix        Save mixture data. [default: False].
+   -f, --ft         Save feature/truth_f data. [default: False].
+   -s, --segsnr     Save segsnr data. [default: False].
 
-Create mixture database data for training and evaluation.
+Create mixture database data for training and evaluation. Optionally, also create mixture audio and feature/truth data.
 
 genmixdb creates a database of training and evaluation feature and truth data generation information. It allows
 the choice of audio neural-network feature types that are supported by the Aaware real-time front-end and truth
@@ -21,9 +24,9 @@ file (also called labels). If you don't already have it, genmixdb can create tru
 energy-based sound detection on each frame of the feature data. You can also select
 different feature types. Here's an example:
 
-genmixdb target_gfr32ts2.yml
+genmixdb target_gfr32ts2
 
-where target_gfr32ts2.yml contains:
+where target_gfr32ts2 contains config.yml with the following inside:
 ---
 feature: gfr32ts2
 
@@ -34,7 +37,7 @@ target_augmentations:
   - normalize: -3.5
 ...
 
-The mixture database is written to a JSON file that inherits the base name of the config file.
+The mixture database is written to a JSON file (mixdb.json) in the same directory that contains the config.yml file.
 
 #### Target data mix with noise and augmentation
 
@@ -44,8 +47,6 @@ where mix_gfr32ts2.yml contains:
 ---
 feature: gfr32ts2
 
-output: data/my_mix.h5
-
 targets:
   - name: data/target.wav
 
@@ -68,8 +69,7 @@ In this example a time-domain mixture is created and feature data is calculated
 specified by 'feature: gfr32ts2'. Various feature types are available which vary in
 spectral and temporal resolution (4 ms or higher), and other feature algorithm
 parameters. The total feature size, dimension, and #frames for mixture is reported
-in the log file (the log file name is derived from the output file base name; in this
-case it would be mix_gfr32ts2.log).
+in the log file (the log file name is genmixdb.log).
 
 Truth (labels) can be automatically created per feature output frame based on sound
 energy detection. By default, these are appended to the feature data in a single HDF5
@@ -111,66 +111,71 @@ targets:
 will find all .wav files in the specified directories and process them as targets.
 
 """
+from dataclasses import dataclass
 from typing import List
 
-import numpy as np
-
 from sonusai import logger
+from sonusai.mixture import AudiosT
 from sonusai.mixture import AugmentedTargets
-from sonusai.mixture import Mixture
+from sonusai.mixture import ListAudiosT
+from sonusai.mixture import Location
+from sonusai.mixture import MRecord
 from sonusai.mixture import MixtureDatabase
 
-# NOTE: multiprocessing dictionary is required for run-time performance; using 'partial' is much slower.
-MP_DICT = dict()
+
+# NOTE: global object is required for run-time performance; using 'partial' is much slower.
+@dataclass
+class MPGlobal:
+    mixdb: MixtureDatabase = None
+    save_mix: bool = None
+    save_ft: bool = None
+    save_segsnr: bool = None
 
 
-def genmixdb(file: str = None,
-             config: dict = None,
+MP_GLOBAL = MPGlobal()
+
+
+def genmixdb(location: Location,
+             save_mix: bool = False,
+             save_ft: bool = False,
+             save_segsnr: bool = False,
              logging: bool = True,
-             show_progress: bool = False) -> MixtureDatabase:
+             show_progress: bool = False,
+             test_mode: bool = False) -> MixtureDatabase:
     from random import seed
-    from typing import List
 
+    import numpy as np
     import yaml
     from tqdm import tqdm
 
     import sonusai
     from sonusai import SonusAIError
+    from sonusai import logger
     from sonusai.mixture import Augmentation
-    from sonusai.mixture import Mixture
-    from sonusai.mixture import MixtureDatabase
-    from sonusai.mixture import Mixtures
+    from sonusai.mixture import MRecord
+    from sonusai.mixture import MixtureDatabaseConfig
+    from sonusai.mixture import SAMPLE_RATE
     from sonusai.mixture import TruthSettings
     from sonusai.mixture import balance_targets
-    from sonusai.mixture import build_noise_audios
     from sonusai.mixture import evaluate_random_rule
+    from sonusai.mixture import generate_mixture_filename
     from sonusai.mixture import get_augmentation_indices_for_mixup
     from sonusai.mixture import get_augmentations
     from sonusai.mixture import get_augmented_target_indices_for_mixup
     from sonusai.mixture import get_augmented_targets
     from sonusai.mixture import get_class_weights_threshold
-    from sonusai.mixture import get_feature_stats
+    from sonusai.mixture import get_ir_files
     from sonusai.mixture import get_mixups
     from sonusai.mixture import get_noise_files
     from sonusai.mixture import get_target_files
-    from sonusai.mixture import get_total_class_count
     from sonusai.mixture import load_config
     from sonusai.mixture import log_duration_and_sizes
-    from sonusai.mixture import read_raw_target_audio
-    from sonusai.mixture import SAMPLE_RATE
     from sonusai.mixture import update_truth_settings
     from sonusai.utils import dataclass_from_dict
     from sonusai.utils import p_tqdm_map
     from sonusai.utils import seconds_to_hms
 
-    if logging:
-        logger.info('')
-
-    if (file is None and config is None) or (file is not None and config is not None):
-        raise SonusAIError(f'Must specify either file name or config')
-
-    if file is not None:
-        config = load_config(file)
+    config = load_config(location)
 
     seed(config['seed'])
 
@@ -187,7 +192,7 @@ def genmixdb(file: str = None,
 
     if logging:
         logger.debug('List of targets:')
-        logger.debug(yaml.dump([sub.name for sub in target_files], default_flow_style=False))
+        logger.debug(yaml.dump([target_file.name for target_file in target_files], default_flow_style=False))
         logger.debug('')
 
     if logging:
@@ -195,12 +200,20 @@ def genmixdb(file: str = None,
     noise_files = get_noise_files(config)
     if logging:
         logger.debug('List of noises:')
-        logger.debug(yaml.dump([sub.name for sub in noise_files], default_flow_style=False))
+        logger.debug(yaml.dump([noise_file.name for noise_file in noise_files], default_flow_style=False))
+        logger.debug('')
+
+    if logging:
+        logger.info('Collecting impulse responses')
+    ir_files = get_ir_files(config)
+    if logging:
+        logger.debug('List of impulse responses:')
+        logger.debug(yaml.dump([ir_file for ir_file in ir_files], default_flow_style=False))
         logger.debug('')
 
     if logging:
         logger.info('Collecting target augmentations')
-    target_augmentations = get_augmentations(rules=config['target_augmentations'])
+    target_augmentations = get_augmentations(rules=config['target_augmentations'], num_ir=len(ir_files))
     mixups = get_mixups(target_augmentations)
     if logging:
         for mixup in mixups:
@@ -212,7 +225,7 @@ def genmixdb(file: str = None,
 
     if logging:
         logger.info('Collecting noise augmentations')
-    noise_augmentations = get_augmentations(config['noise_augmentations'])
+    noise_augmentations = get_augmentations(rules=config['noise_augmentations'], num_ir=len(ir_files))
     if logging:
         logger.debug('Expanded list of noise augmentations:')
         for augmentation in noise_augmentations:
@@ -231,58 +244,41 @@ def genmixdb(file: str = None,
     if truth_mutex and any(mixup > 1 for mixup in mixups):
         raise SonusAIError(f'Mutex truth mode is not compatible with mixup')
 
-    fs = get_feature_stats(feature_mode=config['feature'],
-                           frame_size=config['frame_size'],
-                           num_classes=config['num_classes'],
-                           truth_mutex=truth_mutex)
-
     augmented_targets = get_augmented_targets(target_files, target_augmentations)
 
     mixdb = MixtureDatabase(
-        class_balancing=config['class_balancing'],
-        class_balancing_augmentation=dataclass_from_dict(Augmentation, config['class_balancing_augmentation']),
-        class_labels=config['class_labels'],
-        class_weights_threshold=get_class_weights_threshold(config),
-        exhaustive_noise=config['exhaustive_noise'],
-        feature=config['feature'],
-        feature_samples=fs.feature_samples,
-        feature_step_samples=fs.feature_step_samples,
-        first_cba_index=len(target_augmentations),
-        frame_size=config['frame_size'],
-        noise_augmentations=noise_augmentations,
-        noises=noise_files,
-        num_classes=config['num_classes'],
-        random_snrs=config['random_snrs'],
-        seed=config['seed'],
-        snrs=config['snrs'],
-        target_augmentations=target_augmentations,
-        targets=target_files,
-        truth_mutex=truth_mutex,
-        truth_reduction_function=config['truth_reduction_function'],
-        truth_settings=dataclass_from_dict(TruthSettings, update_truth_settings(config['truth_settings'])),
-    )
+        config=MixtureDatabaseConfig(
+            class_balancing=config['class_balancing'],
+            class_balancing_augmentation=dataclass_from_dict(Augmentation, config['class_balancing_augmentation']),
+            class_labels=config['class_labels'],
+            class_weights_threshold=get_class_weights_threshold(config),
+            exhaustive_noise=config['exhaustive_noise'],
+            feature=config['feature'],
+            first_cba_index=len(target_augmentations),
+            ir_files=ir_files,
+            mixtures=[],
+            noise_augmentations=noise_augmentations,
+            noises=noise_files,
+            num_classes=config['num_classes'],
+            random_snrs=config['random_snrs'],
+            seed=config['seed'],
+            snrs=config['snrs'],
+            target_augmentations=target_augmentations,
+            targets=target_files,
+            truth_mutex=truth_mutex,
+            truth_reduction_function=config['truth_reduction_function'],
+            truth_settings=dataclass_from_dict(TruthSettings, update_truth_settings(config['truth_settings']))),
+        show_progress=show_progress)
+    mixdb.location = location
 
-    MP_DICT['mixdb'] = mixdb
-
-    if logging:
-        logger.info('')
-
-    raw_target_audio = read_raw_target_audio(mixdb=mixdb, show_progress=show_progress)
-    raw_target_audio_samples = sum([len(x) for x in raw_target_audio])
+    raw_target_audio_samples = sum([len(x) for x in mixdb.raw_target_audios])
     if logging:
         logger.info('')
         logger.info(f'Raw target audio: {seconds_to_hms(seconds=raw_target_audio_samples / SAMPLE_RATE)}')
-        logger.info('')
 
-    noise_audios = build_noise_audios(mixdb=mixdb, show_progress=show_progress)
-    noise_audio_samples = sum([len(xa) for x in noise_audios for xa in x])
+    noise_audio_samples = sum([len(xa) for x in mixdb.augmented_noise_audios for xa in x])
     if logging:
-        logger.info('')
         logger.info(f'Augmented noise audio: {seconds_to_hms(seconds=noise_audio_samples / SAMPLE_RATE)}')
-        logger.info('')
-
-    MP_DICT['raw_target_audio'] = raw_target_audio
-    MP_DICT['noise_audios'] = noise_audios
 
     augmented_targets = balance_targets(mixdb, augmented_targets)
 
@@ -300,8 +296,8 @@ def genmixdb(file: str = None,
                 augmentation = target_augmentations[augmented_target.target_augmentation_index]
                 if augmentation.tempo is not None:
                     length /= augmentation.tempo
-                if length % fs.feature_step_samples:
-                    length += fs.feature_step_samples - int(length % fs.feature_step_samples)
+                if length % mixdb.feature_step_samples:
+                    length += mixdb.feature_step_samples - int(length % mixdb.feature_step_samples)
                 total_duration += float(length) / sonusai.mixture.SAMPLE_RATE
 
     total_snrs = len(mixdb.snrs) + len(mixdb.random_snrs)
@@ -310,6 +306,7 @@ def genmixdb(file: str = None,
     noise_sets = len(noise_files) * len(noise_augmentations) if mixdb.exhaustive_noise else 1
     total_duration *= noise_sets
     total_mixtures = noise_sets * target_sets
+    mixid_padding = int(np.ceil(np.log10(total_mixtures)))
     if logging:
         logger.info('')
         logger.info(f'Found {total_mixtures:,} mixtures to process')
@@ -317,26 +314,23 @@ def genmixdb(file: str = None,
     if logging:
         log_duration_and_sizes(total_duration=total_duration,
                                num_classes=mixdb.num_classes,
-                               feature_step_samples=fs.feature_step_samples,
-                               num_bands=fs.num_bands,
-                               stride=fs.stride,
+                               feature_step_samples=mixdb.feature_step_samples,
+                               num_bands=mixdb.fg.num_bands,
+                               stride=mixdb.fg.stride,
                                desc='Estimated')
-        logger.info(f'Feature shape:        {fs.stride} x {fs.num_bands} ({fs.stride * fs.num_bands} total params)')
-        logger.info(f'Feature samples:      {fs.feature_samples} samples ({fs.feature_ms} ms)')
-        logger.info(f'Feature step samples: {fs.feature_step_samples} samples ({fs.feature_step_ms} ms)')
+        logger.info(
+            f'Feature shape:        {mixdb.fg.stride} x {mixdb.fg.num_bands} ({mixdb.fg.stride * mixdb.fg.num_bands} total params)')
+        logger.info(f'Feature samples:      {mixdb.feature_samples} samples ({mixdb.feature_ms} ms)')
+        logger.info(f'Feature step samples: {mixdb.feature_step_samples} samples ({mixdb.feature_step_ms} ms)')
         logger.info('')
 
     used_noise_samples = 0
     if mixdb.exhaustive_noise:
         # Get indices and offsets
-        # mixtures is a nested list: noise, target
-        mixtures: List[Mixtures] = list()
-
         for noise_file_index in range(len(noise_files)):
             for noise_augmentation_index, noise_augmentation in enumerate(noise_augmentations):
-                mixtures.append(list())
                 noise_offset = 0
-                noise_length = len(noise_audios[noise_file_index][noise_augmentation_index])
+                noise_length = len(mixdb.augmented_noise_audios[noise_file_index][noise_augmentation_index])
 
                 for mixup in mixups:
                     augmented_target_indices_for_mixup = get_augmented_target_indices_for_mixup(
@@ -344,58 +338,45 @@ def genmixdb(file: str = None,
                         augmented_targets=augmented_targets,
                         mixup=mixup)
 
-                    for augmented_target_indices_for_mixup in augmented_target_indices_for_mixup:
+                    for augmented_target_index_for_mixup in augmented_target_indices_for_mixup:
                         (target_file_index,
                          target_augmentation_index,
                          target_length) = _get_target_info(
                             mixdb=mixdb,
-                            augmented_target_indices_for_mixup=augmented_target_indices_for_mixup,
+                            augmented_target_indices_for_mixup=augmented_target_index_for_mixup,
                             augmented_targets=augmented_targets,
-                            raw_target_audio=raw_target_audio)
+                            raw_target_audios=mixdb.raw_target_audios)
 
                         for snr in mixdb.snrs:
-                            mixtures[-1].append(Mixture(
+                            mixdb.mixtures.append(MRecord(
+                                name=generate_mixture_filename(len(mixdb.mixtures), mixid_padding),
                                 target_file_index=target_file_index,
                                 target_augmentation_index=target_augmentation_index,
                                 noise_file_index=noise_file_index,
                                 noise_offset=noise_offset,
                                 noise_augmentation_index=noise_augmentation_index,
                                 snr=snr,
-                                random_snr=False,
-                            ))
+                                random_snr=False))
 
                             noise_offset = int((noise_offset + target_length) % noise_length)
                             used_noise_samples += target_length
 
                         for random_snr in mixdb.random_snrs:
-                            mixtures[-1].append(Mixture(
+                            mixdb.mixtures.append(MRecord(
+                                name=generate_mixture_filename(len(mixdb.mixtures), mixid_padding),
                                 target_file_index=target_file_index,
                                 target_augmentation_index=target_augmentation_index,
                                 noise_file_index=noise_file_index,
                                 noise_offset=noise_offset,
                                 noise_augmentation_index=noise_augmentation_index,
                                 snr=evaluate_random_rule(random_snr),
-                                random_snr=True,
-                            ))
+                                random_snr=True))
 
                             noise_offset = int((noise_offset + target_length) % noise_length)
                             used_noise_samples += target_length
 
-        # Fill in the details
-        progress = tqdm(total=total_mixtures, desc='genmixdb', disable=not show_progress)
-        for n_id in range(len(mixtures)):
-            m = mixtures[n_id][0]
-            MP_DICT['augmented_noise_audio'] = noise_audios[m.noise_file_index][m.noise_augmentation_index]
-            mixtures[n_id] = p_tqdm_map(_process_mixture, mixtures[n_id], progress=progress)
-        progress.close()
-
-        # Flatten mixtures
-        mixdb.mixtures = [item for sublist in mixtures for item in sublist]
-
     else:
         # Get indices and offsets
-        mixtures: Mixtures = list()
-
         noise_offset = 0
         noise_file_index = 0
         noise_augmentation_index = 0
@@ -404,32 +385,33 @@ def genmixdb(file: str = None,
                 mixdb=mixdb,
                 augmented_targets=augmented_targets,
                 mixup=mixup)
-            for augmented_target_indices_for_mixup in augmented_target_indices_for_mixup:
+            for augmented_target_index_for_mixup in augmented_target_indices_for_mixup:
                 (target_file_index,
                  target_augmentation_index,
                  target_length) = _get_target_info(
                     mixdb=mixdb,
-                    augmented_target_indices_for_mixup=augmented_target_indices_for_mixup,
+                    augmented_target_indices_for_mixup=augmented_target_index_for_mixup,
                     augmented_targets=augmented_targets,
-                    raw_target_audio=raw_target_audio)
+                    raw_target_audios=mixdb.raw_target_audios)
 
                 for snr in mixdb.snrs:
                     (noise_file_index,
                      noise_augmentation_index,
                      noise_offset) = _get_next_noise_offset(target_length=target_length,
-                                                            noise_audios=noise_audios,
+                                                            augmented_noise_audios=mixdb.augmented_noise_audios,
                                                             noise_file_index=noise_file_index,
                                                             noise_augmentation_index=noise_augmentation_index,
                                                             noise_offset=noise_offset)
 
-                    mixtures.append(Mixture(target_file_index=target_file_index,
-                                            target_augmentation_index=target_augmentation_index,
-                                            noise_file_index=noise_file_index,
-                                            noise_augmentation_index=noise_augmentation_index,
-                                            noise_offset=noise_offset,
-                                            snr=snr,
-                                            random_snr=False,
-                                            ))
+                    mixdb.mixtures.append(MRecord(
+                        name=generate_mixture_filename(len(mixdb.mixtures), mixid_padding),
+                        target_file_index=target_file_index,
+                        target_augmentation_index=target_augmentation_index,
+                        noise_file_index=noise_file_index,
+                        noise_augmentation_index=noise_augmentation_index,
+                        noise_offset=noise_offset,
+                        snr=snr,
+                        random_snr=False))
 
                     noise_offset += target_length
                     used_noise_samples += target_length
@@ -438,66 +420,72 @@ def genmixdb(file: str = None,
                     (noise_file_index,
                      noise_augmentation_index,
                      noise_offset) = _get_next_noise_offset(target_length=target_length,
-                                                            noise_audios=noise_audios,
+                                                            augmented_noise_audios=mixdb.augmented_noise_audios,
                                                             noise_file_index=noise_file_index,
                                                             noise_augmentation_index=noise_augmentation_index,
                                                             noise_offset=noise_offset)
 
-                    mixtures.append(Mixture(target_file_index=target_file_index,
-                                            target_augmentation_index=target_augmentation_index,
-                                            noise_file_index=noise_file_index,
-                                            noise_augmentation_index=noise_augmentation_index,
-                                            noise_offset=noise_offset,
-                                            snr=evaluate_random_rule(random_snr),
-                                            random_snr=True,
-                                            ))
+                    mixdb.mixtures.append(MRecord(
+                        name=generate_mixture_filename(len(mixdb.mixtures), mixid_padding),
+                        target_file_index=target_file_index,
+                        target_augmentation_index=target_augmentation_index,
+                        noise_file_index=noise_file_index,
+                        noise_augmentation_index=noise_augmentation_index,
+                        noise_offset=noise_offset,
+                        snr=evaluate_random_rule(random_snr),
+                        random_snr=True))
 
                     noise_offset += target_length
                     used_noise_samples += target_length
 
-        # Fill in the details
-        progress = tqdm(total=total_mixtures, desc='genmixdb', disable=not show_progress)
-        mixtures = p_tqdm_map(_process_mixture_nen, mixtures, progress=progress)
-        progress.close()
+    # Fill in the details
+    MP_GLOBAL.mixdb = mixdb
+    MP_GLOBAL.save_mix = save_mix
+    MP_GLOBAL.save_ft = save_ft
+    MP_GLOBAL.save_segsnr = save_segsnr
 
-        mixdb.mixtures = mixtures
+    progress = tqdm(total=total_mixtures, desc='genmixdb', disable=not show_progress)
+    mixdb.mixtures = p_tqdm_map(_process_mixture, range(total_mixtures), progress=progress)
+    progress.close()
 
-    mixdb.class_count = get_total_class_count(mixdb)
-
-    total_samples = sum([sub.samples for sub in mixdb.mixtures])
+    total_samples = mixdb.total_samples()
     total_duration = total_samples / sonusai.mixture.SAMPLE_RATE
     if logging:
         log_duration_and_sizes(total_duration=total_duration,
                                num_classes=mixdb.num_classes,
-                               feature_step_samples=fs.feature_step_samples,
-                               num_bands=fs.num_bands,
-                               stride=fs.stride,
+                               feature_step_samples=mixdb.feature_step_samples,
+                               num_bands=mixdb.fg.num_bands,
+                               stride=mixdb.fg.stride,
                                desc='Actual')
         noise_samples_percent = (float(used_noise_samples) / float(noise_audio_samples)) * 100
         logger.info('')
         logger.info(f'Used {noise_samples_percent:,.0f}% of noise audio')
         logger.info('')
 
+    if not test_mode:
+        mixdb.save()
+        logger.info(f'Wrote mixture database to {mixdb.location}')
+
     return mixdb
 
 
 def _get_next_noise_offset(target_length: int,
-                           noise_audios: List[List[np.ndarray]],
+                           augmented_noise_audios: ListAudiosT,
                            noise_file_index: int,
                            noise_augmentation_index: int,
                            noise_offset: int) -> (int, int, int):
     from sonusai import SonusAIError
 
-    if noise_offset + target_length >= len(noise_audios[noise_file_index][noise_augmentation_index]):
+    if noise_offset + target_length >= len(augmented_noise_audios[noise_file_index][noise_augmentation_index]):
         if noise_offset == 0:
             raise SonusAIError('Length of target audio exceeds length of noise audio')
 
         noise_offset = 0
         noise_augmentation_index += 1
-        if noise_augmentation_index == len(noise_audios[noise_file_index]):
+        if noise_augmentation_index == len(augmented_noise_audios[noise_file_index]):
             noise_augmentation_index = 0
             noise_file_index += 1
-            if noise_file_index == len(noise_audios):
+            if noise_file_index == len(augmented_noise_audios):
                 noise_file_index = 0
 
     return noise_file_index, noise_augmentation_index, noise_offset
@@ -506,11 +494,11 @@ def _get_next_noise_offset(target_length: int,
 def _get_target_info(mixdb: MixtureDatabase,
                      augmented_target_indices_for_mixup: List[int],
                      augmented_targets: AugmentedTargets,
-                     raw_target_audio: List[np.ndarray]) -> (List[int], List[int], int):
+                     raw_target_audios: AudiosT) -> (List[int], List[int], int):
     from sonusai.mixture import estimate_audio_length
 
-    target_file_index = list()
-    target_augmentation_index = list()
+    target_file_index = []
+    target_augmentation_index = []
     target_length = 0
     for idx in augmented_target_indices_for_mixup:
         tfi = augmented_targets[idx].target_file_index
@@ -519,84 +507,112 @@ def _get_target_info(mixdb: MixtureDatabase,
         target_file_index.append(tfi)
         target_augmentation_index.append(tai)
 
-        target_audio = raw_target_audio[tfi]
+        raw_target_audio = raw_target_audios[tfi]
         target_augmentation = mixdb.target_augmentations[tai]
-        target_length = max(estimate_audio_length(audio_in=target_audio,
+        target_length = max(estimate_audio_length(audio=raw_target_audio,
                                                   augmentation=target_augmentation,
                                                   length_common_denominator=mixdb.feature_step_samples),
                             target_length)
     return target_file_index, target_augmentation_index, target_length
 
 
-def get_output_from_config(config: dict, config_name: str) -> str:
-    from os.path import splitext
-
+def _process_mixture(mixid: int) -> MRecord:
     from sonusai import SonusAIError
+    from sonusai.mixture import initialize_mixture
 
-    try:
-        config_base = splitext(config_name)[0]
-        name = str(splitext(config['output'])[0])
-        name = name.replace('${config}', config_base)
-        return name
-    except Exception as e:
-        raise SonusAIError(f'Error getting genmixdb base name: {e}')
+    mrecord = MP_GLOBAL.mixdb.mixtures[mixid]
+    if len(mrecord.target_file_index) != len(mrecord.target_augmentation_index):
+        raise SonusAIError('target_file_index and target_augmentation_index are not the same length')
 
+    raw_target_audios = [MP_GLOBAL.mixdb.raw_target_audios[mrecord.target_file_index[ti]] for ti in
+                         range(len(mrecord.target_file_index))]
+    noise_audio = MP_GLOBAL.mixdb.augmented_noise_audios[mrecord.noise_file_index][mrecord.noise_augmentation_index]
 
-def _process_mixture(mixture: Mixture) -> Mixture:
-    from sonusai.mixture import process_mixture
+    mrecord, genmix_data = initialize_mixture(mixdb=MP_GLOBAL.mixdb,
+                                              mrecord=mrecord,
+                                              raw_target_audios=raw_target_audios,
+                                              noise_audio=noise_audio)
 
-    return process_mixture(mixture=mixture,
-                           mixdb=MP_DICT['mixdb'],
-                           raw_target_audios=MP_DICT['raw_target_audio'],
-                           augmented_noise_audio=MP_DICT['augmented_noise_audio'])
+    if MP_GLOBAL.save_mix or MP_GLOBAL.save_ft:
+        write_data = []
 
+        if MP_GLOBAL.save_mix:
+            write_data.append(('targets', genmix_data.targets))
+            write_data.append(('noise', genmix_data.noise))
+            write_data.append(('mixture', genmix_data.mixture))
 
-def _process_mixture_nen(mixture: Mixture) -> Mixture:
-    from sonusai.mixture import process_mixture_nen
+        if MP_GLOBAL.save_ft:
+            truth_t = MP_GLOBAL.mixdb.mixture_truth_t(mixid=mixid,
+                                                      targets=genmix_data.targets,
+                                                      noise=genmix_data.noise)
+            feature, truth_f = MP_GLOBAL.mixdb.mixture_ft(mixid=mixid,
+                                                          mixture=genmix_data.mixture,
+                                                          truth_t=truth_t)
+            write_data.append(('feature', feature))
+            write_data.append(('truth_f', truth_f))
 
-    return process_mixture_nen(mixture=mixture,
-                               mixdb=MP_DICT['mixdb'],
-                               raw_target_audios=MP_DICT['raw_target_audio'],
-                               augmented_noise_audios=MP_DICT['noise_audios'])
+            if MP_GLOBAL.save_segsnr:
+                segsnr = MP_GLOBAL.mixdb.mixture_segsnr(mixid=mixid,
+                                                        targets=genmix_data.targets,
+                                                        noise=genmix_data.noise)
+                write_data.append(('segsnr', segsnr))
+
+        MP_GLOBAL.mixdb.write_mixture_data(mixid, write_data)
+        MP_GLOBAL.mixdb.write_mixture_metadata(mixid)
+
+    return mrecord
 
 
 def main():
-    import time
-
     from docopt import docopt
 
     import sonusai
-    from sonusai import create_file_handler
-    from sonusai import initial_log_messages
-    from sonusai import update_console_handler
-    from sonusai.mixture import load_config
-    from sonusai.utils import seconds_to_hms
     from sonusai.utils import trim_docstring
 
     args = docopt(trim_docstring(__doc__), version=sonusai.__version__, options_first=True)
 
+    import time
+    from os import makedirs
+    from os import remove
+    from os.path import exists
+    from os.path import isdir
+    from os.path import join
+
+    from sonusai import create_file_handler
+    from sonusai import initial_log_messages
+    from sonusai import logger
+    from sonusai import update_console_handler
+    from sonusai.utils import seconds_to_hms
+
     verbose = args['--verbose']
+    save_mix = args['--mix']
+    save_ft = args['--ft']
+    save_segsnr = args['--segsnr']
+    location = args['LOC']
 
-    for config_file in args['CONFIG']:
-        start_time = time.monotonic()
-        logger.info(f'Creating mixture database for {config_file}')
-        config = load_config(config_file)
-        output = get_output_from_config(config, config_file)
+    start_time = time.monotonic()
 
-        log_name = output + '.log'
-        create_file_handler(log_name)
-        update_console_handler(verbose)
-        initial_log_messages('genmixdb')
+    if exists(location) and not isdir(location):
+        remove(location)
 
-        mixdb = genmixdb(config=config, show_progress=True)
+    makedirs(location, exist_ok=True)
 
-        json_name = output + '.json'
-        with open(file=json_name, mode='w') as file:
-            file.write(mixdb.to_json(indent=2))
-            logger.info(f'Wrote mixture database for {config_file} to {json_name}')
+    create_file_handler(join(location, 'genmixdb.log'))
+    update_console_handler(verbose)
+    initial_log_messages('genmixdb')
 
-        end_time = time.monotonic()
-        logger.info(f'Completed in {seconds_to_hms(seconds=end_time - start_time)}')
+    logger.info(f'Creating mixture database for {location}')
+    logger.info('')
+
+    genmixdb(location=location,
+             save_mix=save_mix,
+             save_ft=save_ft,
+             save_segsnr=save_segsnr,
+             show_progress=True)
+
+    end_time = time.monotonic()
+    logger.info(f'Completed in {seconds_to_hms(seconds=end_time - start_time)}')
+    logger.info('')
 
 
 if __name__ == '__main__':
