@@ -3,19 +3,21 @@
 usage: sonusai [--version] [--help] <command> [<args>...]
 
 The sonusai commands are:
-   evaluate         Evaluate model performance
-   genft            Generate feature and truth data
-   genmix           Generate mixture and truth data
-   genmixdb         Generate a mixture database
-   gentcst          Generate target configuration from a subdirectory tree
-   keras_predict    Run Keras predict on a trained model
-   keras_train      Train a model using Keras
-   lsdb             List information about a mixture database
-   mkwav            Make WAV files from a mixture database
-   predict          Run predict on a trained model
-   plot             Plot mixture data
-   tplot            Plot truth data
-   vars             List custom SonusAI variables
+   calc_metric_spenh_targetf    Run speech enhancement and analysis for targetf truth
+   evaluate                     Evaluate model performance
+   genft                        Generate feature and truth data
+   genmix                       Generate mixture and truth data
+   genmixdb                     Generate a mixture database
+   gentcst                      Generate target configuration from a subdirectory tree
+   keras_predict                Run Keras predict on a trained model
+   keras_train                  Train a model using Keras
+   keras_onnx                   Convert a trained Keras model to ONNX
+   lsdb                         List information about a mixture database
+   mkwav                        Make WAV files from a mixture database
+   onnx_predict                 Run ONNX predict on a trained model
+   plot                         Plot mixture data
+   tplot                        Plot truth data
+   vars                         List custom SonusAI variables
 
 Aaware Sound and Voice Machine Learning Framework. See 'sonusai help <command>'
 for more information on a specific command.
@@ -31,10 +33,11 @@ def main():
     from docopt import docopt
 
     import sonusai
-    from sonusai import logger
+    from sonusai import SonusAIError
     from sonusai.utils import trim_docstring
 
     commands = (
+        'calc_metric_spenh_targetf',
         'evaluate',
         'genft',
         'genmix',
@@ -42,10 +45,11 @@ def main():
         'gentcst',
         'keras_predict',
         'keras_train',
+        'keras_onnx',
         'lsdb',
         'mkwav',
+        'onnx_predict',
         'plot',
-        'predict',
         'tplot',
         'vars',
     )
@@ -61,13 +65,11 @@ def main():
         elif argv[0] in commands:
             exit(call(['python', f'{sonusai.BASEDIR}/{argv[0]}.py', '-h']))
         else:
-            logger.error(f"{argv[0]} is not a SonusAI command. See 'sonusai help'.")
-            raise SystemExit(1)
+            raise SonusAIError(f"{argv[0]} is not a SonusAI command. See 'sonusai help'.")
     elif command in commands:
         exit(call(['python', f'{sonusai.BASEDIR}/{command}.py'] + argv))
 
-    logger.error(f"{command} is not a SonusAI command. See 'sonusai help'.")
-    raise SystemExit(1)
+    raise SonusAIError(f"{command} is not a SonusAI command. See 'sonusai help'.")
 
 
 if __name__ == '__main__':
