@@ -4,14 +4,17 @@ import numpy as np
 import pandas as pd
 
 from sonusai.mixture.mixdb import MixtureDatabase
-from sonusai.mixture.mixdb import MixtureID
+from sonusai.mixture.types import GeneralizedIDs
+from sonusai.mixture.types import Predict
+from sonusai.mixture.types import Segsnr
+from sonusai.mixture.types import Truth
 
 
 def snr_summary(mixdb: MixtureDatabase,
-                mixid: MixtureID,
-                truth_f: np.ndarray,
-                predict: np.ndarray,
-                segsnr: Union[np.ndarray, None] = None,
+                mixid: GeneralizedIDs,
+                truth_f: Truth,
+                predict: Predict,
+                segsnr: Segsnr = None,
                 predict_thr: Union[float, np.ndarray] = 0,
                 truth_thr: float = 0.5,
                 timesteps: int = 0) -> (pd.DataFrame, pd.DataFrame, pd.DataFrame, dict):
@@ -19,9 +22,9 @@ def snr_summary(mixdb: MixtureDatabase,
        Inputs:
          mixdb        Mixture database
          mixid
-         truth_f      Truth/labels (features, num_classes)
-         predict      Prediction data / neural net model one-hot out (features, num_classes)
-         segsnr       Segmental SNR from SonusAI genft  (transform_frames, 1)
+         truth_f      Truth/labels [features, num_classes]
+         predict      Prediction data / neural net model one-hot out [features, num_classes]
+         segsnr       Segmental SNR from SonusAI genft  [transform_frames, 1]
          predict_thr  Decision threshold(s) applied to predict data, allowing predict to be
                       continuous probabilities or decisions
          truth_thr    Decision threshold(s) applied to truth data, allowing truth to be
@@ -37,12 +40,11 @@ def snr_summary(mixdb: MixtureDatabase,
     import warnings
 
     from sonusai.metrics import one_hot
-    from sonusai.mixture import get_mixid_data
     from sonusai.queries import get_mixids_from_snr
 
     num_classes = truth_f.shape[1]
 
-    snr_mixids = get_mixids_from_snr(mixdb=mixdb, mixid=mixid)
+    snr_mixids = get_mixids_from_snr(mixdb=mixdb, mixids=mixid)
 
     # Check predict_thr array or scalar and return final scalar predict_thr value
     if not mixdb.truth_mutex and num_classes > 1:
@@ -59,9 +61,9 @@ def snr_summary(mixdb: MixtureDatabase,
                     # multi-label predict_thr array set to scalar = array[0]
                     predict_thr = predict_thr[0]
 
-    macro_avg = np.zeros((len(snr_mixids), 7), dtype=float)
-    micro_avg = np.zeros((len(snr_mixids), 7), dtype=float)
-    wghtd_avg = np.zeros((len(snr_mixids), 7), dtype=float)
+    macro_avg = np.zeros((len(snr_mixids), 7), dtype=np.float32)
+    micro_avg = np.zeros((len(snr_mixids), 7), dtype=np.float32)
+    wghtd_avg = np.zeros((len(snr_mixids), 7), dtype=np.float32)
     ssnr_stats = None
     segsnr_f = None
 
@@ -70,11 +72,11 @@ def snr_summary(mixdb: MixtureDatabase,
         # expected to always be an integer
         feature_frames = int(segsnr.shape[0] / truth_f.shape[0])
         segsnr_f = np.mean(np.reshape(segsnr, (truth_f.shape[0], feature_frames)), axis=1, keepdims=True)
-        ssnr_stats = np.zeros((len(snr_mixids), 3), dtype=float)
+        ssnr_stats = np.zeros((len(snr_mixids), 3), dtype=np.float32)
 
     ii = 0
     for snr in snr_mixids:
-        y_truth, y_predict = get_mixid_data(mixdb, snr_mixids[snr], truth_f, predict)
+        y_truth, y_predict = get_mixids_data(mixdb, snr_mixids[snr], truth_f, predict)
         _, metrics, _, _, _, mavg = one_hot(y_truth, y_predict, predict_thr, truth_thr, timesteps)
 
         # mavg macro, micro, weighted: [PPV, TPR, F1, FPR, ACC, mAP, mAUC, TPSUM]
@@ -82,13 +84,13 @@ def snr_summary(mixdb: MixtureDatabase,
         micro_avg[ii, :] = mavg[1, 0:7]
         wghtd_avg[ii, :] = mavg[2, 0:7]
         if segsnr is not None:
-            y_truth, y_segsnr = get_mixid_data(mixdb, snr_mixids[snr], truth_f, segsnr_f)
+            y_truth, y_segsnr = get_mixids_data(mixdb, snr_mixids[snr], truth_f, segsnr_f)
             with warnings.catch_warnings():
                 warnings.filterwarnings(action='ignore', message='divide by zero encountered in log10')
                 # segmental SNR mean = mixture_snr and target_snr
                 ssnr_stats[ii, 0] = 10 * np.log10(np.mean(y_segsnr))
                 # segmental SNR 80% percentile
-                ssnr_stats[ii, 1] = 10 * np.log10(np.percentile(y_segsnr, 80, interpolation='midpoint'))
+                ssnr_stats[ii, 1] = 10 * np.log10(np.percentile(y_segsnr, 80, method='midpoint'))
                 # segmental SNR max
                 ssnr_stats[ii, 2] = 10 * np.log10(max(y_segsnr))
 
