@@ -1,35 +1,33 @@
+from pathlib import Path
 from typing import Dict
 from typing import List
 from typing import Union
 
-from sonusai.mixture.mixdb import NoiseFiles
-from sonusai.mixture.mixdb import TargetFiles
+from sonusai.mixture.types import ImpulseResponseFiles
+from sonusai.mixture.types import Location
+from sonusai.mixture.types import NoiseFiles
+from sonusai.mixture.types import TargetFiles
 
 
-def raw_load_config(name: str) -> dict:
-    """Load YAML file with SonusAI variable substitution."""
+def raw_load_config(name: Location) -> Dict:
+    """Load YAML config file
+
+    :param name: File name
+    :return: Dictionary of config data
+    """
     import yaml
 
     with open(file=name, mode='r') as f:
         config = yaml.safe_load(f)
 
-    return config_variable_substitution(config)
-
-
-def config_variable_substitution(config: dict) -> dict:
-    """Find custom SonusAI variables in given dictionary and substitute their values in place."""
-    import json
-
-    from sonusai.mixture import DEFAULT_FRAME_SIZE
-
-    result = json.dumps(config)
-    result = result.replace('"${frame_size}"', str(DEFAULT_FRAME_SIZE))
-    config = json.loads(result)
     return config
 
 
-def get_default_config() -> dict:
-    """Load default SonusAI config."""
+def get_default_config() -> Dict:
+    """Load default SonusAI config
+
+    :return: Dictionary of default config data
+    """
     from sonusai import SonusAIError
     from sonusai.mixture import DEFAULT_CONFIG
 
@@ -39,13 +37,24 @@ def get_default_config() -> dict:
         raise SonusAIError(f'Error loading default config: {e}')
 
 
-def load_config(name: str) -> dict:
-    """Load SonusAI default config and update with given YAML file (performing SonusAI variable substitution)."""
-    return update_config_from_file(name=name, config=get_default_config())
+def load_config(name: Location) -> Dict:
+    """Load SonusAI default config and update with given location (performing SonusAI variable substitution)
+
+    :param name: Directory containing mixture database
+    :return: Dictionary of config data
+    """
+    from os.path import join
+
+    return update_config_from_file(name=join(name, 'config.yml'), config=get_default_config())
 
 
-def update_config_from_file(name: str, config: dict) -> dict:
-    """Update the given config with the config in the given YAML file."""
+def update_config_from_file(name: Location, config: Dict) -> Dict:
+    """Update the given config with the config in the given YAML file
+
+    :param name: File name
+    :param config: Config dictionary to update
+    :return: Updated config dictionary
+    """
     from copy import deepcopy
 
     from sonusai import SonusAIError
@@ -93,7 +102,12 @@ def update_config_from_file(name: str, config: dict) -> dict:
 
 
 def update_truth_settings(given: Union[List[Dict], Dict], default: List[Dict] = None) -> List[Dict]:
-    """Update missing fields in given 'truth_settings' with default values."""
+    """Update missing fields in given 'truth_settings' with default values
+
+    :param given: The dictionary of given truth settings
+    :param default: The dictionary of default truth settings
+    :return: Updated dictionary of truth settings
+    """
     from copy import deepcopy
 
     from sonusai import SonusAIError
@@ -126,11 +140,15 @@ def update_truth_settings(given: Union[List[Dict], Dict], default: List[Dict] =
     return truth_settings
 
 
-def get_hierarchical_config_files(root: str, leaf: str) -> list:
-    """Get a hierarchical list of config files in the given leaf of the given root."""
+def get_hierarchical_config_files(root: str, leaf: str) -> List[Path]:
+    """Get a hierarchical list of config files in the given leaf of the given root
+
+    :param root: Root of the hierarchy
+    :param leaf: Leaf under the root
+    :return: List of config files found in the hierarchy
+    """
     from sonusai import SonusAIError
     import os
-    from pathlib import Path
 
     config_file = 'config.yml'
 
@@ -151,7 +169,7 @@ def get_hierarchical_config_files(root: str, leaf: str) -> list:
         raise SonusAIError(f'Could not find {top_config_file}')
 
     current = leaf_path
-    config_files = list()
+    config_files = []
     while current != root_path:
         local_config_file = Path(os.path.join(current, config_file))
         if local_config_file.is_file():
@@ -163,7 +181,13 @@ def get_hierarchical_config_files(root: str, leaf: str) -> list:
 
 
 def update_config_from_hierarchy(root: str, leaf: str, config: dict) -> dict:
-    """Update the given config using the hierarchical config files in the given leaf of the given root."""
+    """Update the given config using the hierarchical config files in the given leaf of the given root
+
+    :param root: Root of the hierarchy
+    :param leaf: Leaf under the root
+    :param config: Config to update
+    :return: Updated config
+    """
     from copy import deepcopy
 
     new_config = deepcopy(config)
@@ -175,7 +199,12 @@ def update_config_from_hierarchy(root: str, leaf: str, config: dict) -> dict:
 
 
 def get_max_class(num_classes: int, truth_mutex: bool) -> int:
-    """Get the maximum class index."""
+    """Get the maximum class index
+
+    :param num_classes: Number of classes
+    :param truth_mutex: Truth is mutex mode
+    :return: Highest class index
+    """
     max_class = num_classes
     if truth_mutex:
         max_class -= 1
@@ -183,13 +212,19 @@ def get_max_class(num_classes: int, truth_mutex: bool) -> int:
 
 
 def get_target_files(config: dict) -> TargetFiles:
+    """Get the list of target files from a config
+
+    :param config: Config dictionary
+    :return: List of target files
+    """
     from sonusai import SonusAIError
     from sonusai.utils import dataclass_from_dict
 
     truth_settings = config.get('truth_settings', list())
-    target_files = list()
+    target_level_type = config.get('target_level_type', None)
+    target_files = []
     for target_file in config['targets']:
-        append_target_files(target_files, target_file, truth_settings)
+        append_target_files(target_files, target_file, truth_settings, target_level_type)
 
     max_class = get_max_class(config['num_classes'], config['truth_mode'] == 'mutex')
 
@@ -206,7 +241,16 @@ def get_target_files(config: dict) -> TargetFiles:
 def append_target_files(target_files: List[dict],
                         target_file: Union[dict, str],
                         truth_settings: List[dict],
-                        tokens: Union[dict, None] = None) -> None:
+                        target_level_type: str,
+                        tokens: dict = None) -> None:
+    """Process target files list and append as needed
+
+    :param target_files: Working list of target files
+    :param target_file: Target file entry to append to the list
+    :param truth_settings: Truth settings for the target
+    :param target_level_type: Target level type for the target
+    :param tokens: Tokens used for variable expansion
+    """
     from glob import glob
     from os import listdir
     from os.path import dirname
@@ -220,7 +264,7 @@ def append_target_files(target_files: List[dict],
     from sonusai import SonusAIError
 
     if tokens is None:
-        tokens = dict()
+        tokens = {}
 
     if isinstance(target_file, dict):
         if 'name' in target_file:
@@ -230,6 +274,8 @@ def append_target_files(target_files: List[dict],
 
         if 'truth_settings' in target_file:
             truth_settings = target_file['truth_settings']
+        if 'target_level_type' in target_file:
+            target_level_type = target_file['target_level_type']
     else:
         in_name = target_file
 
@@ -246,7 +292,7 @@ def append_target_files(target_files: List[dict],
                 child = file
                 if not isabs(child):
                     child = join(dir_name, child)
-                append_target_files(target_files, child, truth_settings, tokens)
+                append_target_files(target_files, child, truth_settings, target_level_type, tokens)
         else:
             try:
                 if ext == '.txt':
@@ -260,14 +306,14 @@ def append_target_files(target_files: List[dict],
                                 tokens.update(new_tokens)
                                 if not isabs(child):
                                     child = join(dir_name, child)
-                                append_target_files(target_files, child, truth_settings, tokens)
+                                append_target_files(target_files, child, truth_settings, target_level_type, tokens)
                 elif ext == '.yml':
                     try:
                         yml_config = raw_load_config(name)
 
                         if 'targets' in yml_config:
                             for record in yml_config['targets']:
-                                append_target_files(target_files, record, truth_settings, tokens)
+                                append_target_files(target_files, record, truth_settings, target_level_type, tokens)
                     except Exception as e:
                         raise SonusAIError(f'Error processing {name}: {e}')
                 else:
@@ -276,7 +322,7 @@ def append_target_files(target_files: List[dict],
                     for key, value in tokens.items():
                         name = name.replace(value, f'${key}')
                     entry = {
-                        'name':     name,
+                        'name': name,
                         'duration': duration,
                     }
                     if len(truth_settings) > 0:
@@ -284,6 +330,8 @@ def append_target_files(target_files: List[dict],
                         for truth_setting in entry['truth_settings']:
                             if 'function' in truth_setting and truth_setting['function'] == 'file':
                                 truth_setting['config']['file'] = splitext(name)[0] + '.h5'
+                    if target_level_type is not None:
+                        entry['target_level_type'] = target_level_type
                     target_files.append(entry)
             except SonusAIError:
                 raise
@@ -292,9 +340,14 @@ def append_target_files(target_files: List[dict],
 
 
 def get_noise_files(config: dict) -> NoiseFiles:
+    """Get the list of noise files from a config
+
+    :param config: Config dictionary
+    :return: List of noise file
+    """
     from sonusai.utils import dataclass_from_dict
 
-    noise_files = list()
+    noise_files = []
     for noise_file in config['noises']:
         append_noise_files(noise_files, noise_file)
 
@@ -303,7 +356,13 @@ def get_noise_files(config: dict) -> NoiseFiles:
 
 def append_noise_files(noise_files: List[dict],
                        noise_file: Union[dict, str],
-                       tokens: Union[dict, None] = None) -> None:
+                       tokens: dict = None) -> None:
+    """Process noise files list and append as needed
+
+    :param noise_files: Working list of noise files
+    :param noise_file: Noise file entry to append to the list
+    :param tokens: Tokens used for variable expansion
+    """
     from glob import glob
     from os import listdir
     from os.path import dirname
@@ -317,7 +376,7 @@ def append_noise_files(noise_files: List[dict],
     from sonusai import SonusAIError
 
     if tokens is None:
-        tokens = dict()
+        tokens = {}
 
     if isinstance(noise_file, dict):
         if 'name' in noise_file:
@@ -370,7 +429,7 @@ def append_noise_files(noise_files: List[dict],
                     for key, value in tokens.items():
                         name = name.replace(value, f'${key}')
                     entry = {
-                        'name':     name,
+                        'name': name,
                         'duration': duration,
                     }
                     noise_files.append(entry)
@@ -380,22 +439,108 @@ def append_noise_files(noise_files: List[dict],
                 raise SonusAIError(f'Error processing {name}: {e}')
 
 
+def get_ir_files(config: dict) -> ImpulseResponseFiles:
+    """Get the list of impulse response files from a config
+
+    :param config: Config dictionary
+    :return: List of impulse response files
+    """
+    ir_files = []
+    for ir_file in config['impulse_responses']:
+        append_ir_files(ir_files, ir_file)
+
+    return ir_files
+
+
+def append_ir_files(ir_files: List[Location],
+                    ir_file: Location,
+                    tokens: dict = None) -> None:
+    """Process impulse response files list and append as needed
+
+    :param ir_files: Working list of impulse response files
+    :param ir_file: Impulse response file entry to append to the list
+    :param tokens: Tokens used for variable expansion
+    """
+    from glob import glob
+    from os import listdir
+    from os.path import dirname
+    from os.path import isabs
+    from os.path import isdir
+    from os.path import join
+    from os.path import splitext
+
+    import sox
+
+    from sonusai import SonusAIError
+
+    if tokens is None:
+        tokens = {}
+
+    in_name, new_tokens = tokenized_expandvars(ir_file)
+    tokens.update(new_tokens)
+    names = glob(in_name)
+    if not names:
+        raise SonusAIError(f'Could not find {in_name}. Make sure path exists')
+    for name in names:
+        ext = splitext(name)[1].lower()
+        dir_name = dirname(name)
+        if isdir(name):
+            for file in listdir(name):
+                child = file
+                if not isabs(child):
+                    child = join(dir_name, child)
+                append_ir_files(ir_files, child, tokens)
+        else:
+            try:
+                if ext == '.txt':
+                    with open(file=name, mode='r') as txt_file:
+                        for line in txt_file:
+                            # strip comments
+                            child = line.partition('#')[0]
+                            child = child.rstrip()
+                            if child:
+                                child, new_tokens = tokenized_expandvars(child)
+                                tokens.update(new_tokens)
+                                if not isabs(child):
+                                    child = join(dir_name, child)
+                                append_ir_files(ir_files, child, tokens)
+                elif ext == '.yml':
+                    try:
+                        yml_config = raw_load_config(name)
+
+                        if 'impulse_responses' in yml_config:
+                            for record in yml_config['impulse_responses']:
+                                append_ir_files(ir_files, record, tokens)
+                    except Exception as e:
+                        raise SonusAIError(f'Error processing {name}: {e}')
+                else:
+                    sox.file_info.validate_input_file(name)
+                    for key, value in tokens.items():
+                        name = name.replace(value, f'${key}')
+                    ir_files.append(name)
+            except SonusAIError:
+                raise
+            except Exception as e:
+                raise SonusAIError(f'Error processing {name}: {e}')
+
+
 def tokenized_expandvars(path: str) -> (str, dict):
     """Expand shell variables of the forms $var, ${var} and %var%.
-    Unknown variables are left unchanged."""
+    Unknown variables are left unchanged.
 
-    # Expand paths containing shell variable substitutions.
-    # The following rules apply:
-    #       - no expansion within single quotes
-    #       - '$$' is translated into '$'
-    #       - '%%' is translated into '%' if '%%' are not seen in %var1%%var2%
-    #       - ${varname} is accepted.
-    #       - $varname is accepted.
-    #       - %varname% is accepted.
-    #       - varnames can be made out of letters, digits and the characters '_-'
-    #         (though is not verified in the ${varname} and %varname% cases)
-    # XXX With COMMAND.COM you can use any characters in a variable name,
-    # XXX except '^|<>='.
+    Expand paths containing shell variable substitutions. The following rules apply:
+        - no expansion within single quotes
+        - '$$' is translated into '$'
+        - '%%' is translated into '%' if '%%' are not seen in %var1%%var2%
+        - ${varname} is accepted.
+        - $varname is accepted.
+        - %varname% is accepted.
+        - varnames can be made out of letters, digits and the characters '_-'
+        (though is not verified in the ${varname} and %varname% cases)
+
+    :param path: String to expand
+    :return: Tuple of (expanded string, dictionary of tokens)
+    """
     import os
     import string
 
@@ -508,3 +653,36 @@ def tokenized_expandvars(path: str) -> (str, dict):
             res += c
         index += 1
     return res, token_map
+
+
+def get_class_weights_threshold(config: Dict) -> List[float]:
+    """Get the class_weights_threshold from a config
+
+    :param config: Config dictionary
+    :return: class_weights_threshold
+    """
+    from sonusai import SonusAIError
+
+    class_weights_threshold = config['class_weights_threshold']
+    num_classes = config['num_classes']
+
+    if not isinstance(class_weights_threshold, list):
+        class_weights_threshold = [class_weights_threshold]
+
+    if len(class_weights_threshold) == 1:
+        class_weights_threshold = [class_weights_threshold[0]] * num_classes
+
+    if len(class_weights_threshold) != num_classes:
+        raise SonusAIError(f'invalid class_weights_threshold length: {len(class_weights_threshold)}')
+
+    return class_weights_threshold
+
+
+def generate_mixture_filename(mixid: int, padding: int) -> str:
+    """Generate a zero-padded mixture file name
+
+    :param mixid: Mixture ID
+    :param padding: Amount of padding
+    :return: Zero-padded mixture file name
+    """
+    return f'{mixid:0{padding}}.h5'
