@@ -10,7 +10,7 @@ REQUIRED_CONFIGS = [
     'class_weights_threshold',
     'exhaustive_noise',
     'feature',
-    'frame_size',
+    'impulse_responses',
     'noise_augmentations',
     'noises',
     'num_classes',
@@ -18,6 +18,7 @@ REQUIRED_CONFIGS = [
     'seed',
     'snrs',
     'target_augmentations',
+    'target_level_type',
     'targets',
     'truth_mode',
     'truth_reduction_function',
@@ -25,14 +26,14 @@ REQUIRED_CONFIGS = [
 ]
 VALID_CONFIGS = REQUIRED_CONFIGS + ['output']
 VALID_TRUTH_SETTINGS = ['function', 'config', 'index']
-VALID_AUGMENTATIONS = ['normalize', 'gain', 'pitch', 'tempo', 'eq1', 'eq2', 'eq3', 'lpf', 'count', 'mixup']
+VALID_AUGMENTATIONS = ['normalize', 'gain', 'pitch', 'tempo', 'eq1', 'eq2', 'eq3', 'lpf', 'ir', 'count', 'mixup']
 RAND_PATTERN = re.compile(r'rand\(([-+]?(\d+(\.\d*)?|\.\d+)),\s*([-+]?(\d+(\.\d*)?|\.\d+))\)')
 SAMPLE_RATE = 16000
-BIT_DEPTH = 16
+BIT_DEPTH = 32
+ENCODING = 'floating-point'
 CHANNEL_COUNT = 1
-SAMPLE_BYTES = int(BIT_DEPTH / 8)
+SAMPLE_BYTES = BIT_DEPTH // 8
 FLOAT_BYTES = 4
-DEFAULT_FRAME_SIZE = 64
 
 DEFAULT_NOISE = os.path.join(sonusai.BASEDIR, 'data', 'whitenoise.wav')
 DEFAULT_CONFIG = os.path.join(sonusai.BASEDIR, 'data', 'genmixdb.yml')
