@@ -5,14 +5,14 @@ def log_duration_and_sizes(total_duration: int,
                            stride: int,
                            desc: str) -> None:
     from sonusai import logger
-    from sonusai.mixture import BIT_DEPTH
     from sonusai.mixture import FLOAT_BYTES
+    from sonusai.mixture import SAMPLE_BYTES
     from sonusai.mixture import SAMPLE_RATE
     from sonusai.utils import human_readable_size
     from sonusai.utils import seconds_to_hms
 
     total_samples = total_duration * SAMPLE_RATE
-    mixture_bytes = total_samples * BIT_DEPTH / 8
+    mixture_bytes = total_samples * SAMPLE_BYTES
     truth_t_bytes = total_samples * num_classes * FLOAT_BYTES
     feature_bytes = total_samples / feature_step_samples * stride * num_bands * FLOAT_BYTES
     truth_f_bytes = total_samples / feature_step_samples * num_classes * FLOAT_BYTES
