@@ -1,29 +1,25 @@
 """sonusai plot
 
-usage: plot [-hve] [-m MODEL] [-l CSV] [-i MIXID] [-o OUTPUT] INPUT
+usage: plot [-hve] [-i MIXID] [-m MODEL] [-l CSV] [-o OUTPUT] INPUT
 
 options:
-   -h, --help
-   -v, --verbose                Be verbose.
-   -m MODEL, --model MODEL      Trained model ONNX file.
-   -i MIXID, --mixid MIXID      Mixture to plot if input is HDF5.
-   -l CSV, --labels CSV         Optional CSV file of class labels (from SonusAI gentcst).
-   -o OUTPUT, --output OUTPUT   Optional output HDF5 file for prediction.
-   -e, --energy                 Use energy plots.
+    -h, --help
+    -v, --verbose               Be verbose.
+    -i MIXID, --mixid MIXID     Mixture to plot if input is a mixture database.
+    -m MODEL, --model MODEL     Trained model ONNX file.
+    -l CSV, --labels CSV        Optional CSV file of class labels (from SonusAI gentcst).
+    -o OUTPUT, --output OUTPUT  Optional output HDF5 file for prediction.
+    -e, --energy                Use energy plots.
 
-Plot SonusAI audio, feature, truth, and prediction data. INPUT must be one of three file types:
+Plot SonusAI audio, feature, truth, and prediction data. INPUT must be one of the following:
 
     * WAV
       Using the given model, generate feature data and run prediction. A model file must be
       provided. The MIXID is ignored. If --energy is specified, plot predict data as energy.
 
-    * JSON
-      Using the given mixture database, generate feature and truth data. Run prediction if
-      a model is given. The MIXID is required. (--energy is ignored.)
-
-    * HDF5
-      Using the given mixture database, generate feature and truth data. Run prediction if
-      a model is given. The MIXID is required. (--energy is ignored.)
+    * directory
+      Using the given SonusAI mixture database directory, generate feature and truth data if not found.
+      Run prediction if a model is given. The MIXID is required. (--energy is ignored.)
 
 Prediction data will be written to OUTPUT if a model file is given and OUTPUT is specified.
 
@@ -36,9 +32,7 @@ Inputs:
     MODEL   A SonusAI trained ONNX model file. If a model file is given, prediction data will be
             generated.
     INPUT   A WAV file, or
-            a JSON file containing a SonusAI mixture database, or
-            an HDF5 file containing:
-                attribute:  mixdb (required)
+            a directory containing a SonusAI mixture database
 
 Outputs:
     {INPUT}-plot.pdf or {INPUT}-mix{MIXID}-plot.pdf
@@ -47,30 +41,38 @@ Outputs:
 
 """
 from typing import List
-from typing import Union
 
-import matplotlib.pyplot as plt
 import numpy as np
+from matplotlib import pyplot as plt
 
 from sonusai import logger
+from sonusai.mixture import AudioT
+from sonusai.mixture import Feature
+from sonusai.mixture import Predict
+from sonusai.mixture import Truth
 
 
-def spec_plot(mixture: np.ndarray,
-              feature: np.ndarray,
-              predict: Union[np.ndarray, None] = None,
-              labels: Union[List[str], None] = None,
+def spec_plot(mixture: AudioT,
+              feature: Feature,
+              predict: Predict = None,
+              target: AudioT = None,
+              labels: List[str] = None,
               title: str = '') -> plt.figure:
     from sonusai.mixture import SAMPLE_RATE
-    from sonusai.utils import int16_to_float
 
     num_plots = 4 if predict is not None else 2
     fig, ax = plt.subplots(num_plots, 1, constrained_layout=True, figsize=(11, 8.5))
 
     # Plot the waveform
     x_axis = np.arange(len(mixture), dtype=np.float32) / SAMPLE_RATE
-    ax[0].plot(x_axis, int16_to_float(mixture), label='Mixture')
+    ax[0].plot(x_axis, mixture, label='Mixture')
     ax[0].set_xlim(x_axis[0], x_axis[-1])
     ax[0].set_ylim([-1.025, 1.025])
+    if target is not None:
+        # Plot target time-domain waveform on top of mixture
+        color = 'tab:blue'
+        ax[0].plot(x_axis, target, color=color, label='Target')
+        ax[0].set_ylabel('magnitude', color=color)
     ax[0].set_title('Waveform')
 
     # Plot the spectrogram
@@ -99,12 +101,11 @@ def spec_plot(mixture: np.ndarray,
     return fig
 
 
-def spec_energy_plot(mixture: np.ndarray,
-                     feature: np.ndarray,
-                     truth_f: Union[np.ndarray, None] = None,
-                     predict: Union[np.ndarray, None] = None) -> plt.figure:
+def spec_energy_plot(mixture: AudioT,
+                     feature: Feature,
+                     truth_f: Truth = None,
+                     predict: Predict = None) -> plt.figure:
     from sonusai.mixture import SAMPLE_RATE
-    from sonusai.utils import int16_to_float
 
     num_plots = 2
     if truth_f is not None:
@@ -117,7 +118,7 @@ def spec_energy_plot(mixture: np.ndarray,
     # Plot the waveform
     p = 0
     x_axis = np.arange(len(mixture), dtype=np.float32) / SAMPLE_RATE
-    ax[p].plot(x_axis, int16_to_float(mixture), label='Mixture')
+    ax[p].plot(x_axis, mixture, label='Mixture')
     ax[p].set_xlim(x_axis[0], x_axis[-1])
     ax[p].set_ylim([-1.025, 1.025])
     ax[p].set_title('Waveform')
@@ -140,26 +141,25 @@ def spec_energy_plot(mixture: np.ndarray,
     return fig
 
 
-def class_plot(mixture: np.ndarray,
-               target: Union[np.ndarray, None] = None,
-               truth_f: Union[np.ndarray, None] = None,
-               predict: Union[np.ndarray, None] = None,
+def class_plot(mixture: AudioT,
+               target: AudioT = None,
+               truth_f: Truth = None,
+               predict: Predict = None,
                label: str = '') -> plt.figure:
-    """
-    Plot mixture waveform with optional prediction and/or truth together in a single plot.
+    """ Plot mixture waveform with optional prediction and/or truth together in a single plot
+
     The target waveform can optionally be provided, and prediction and truth can have multiple classes.
 
     Inputs:
-      mixture       required, numpy array (samples, 1)
-      target        optional, list of numpy arrays (samples, 1)
-      truth_f       optional, numpy array (frames, 1)
-      predict       optional, numpy array (frames, 1)
+      mixture       required, numpy array [samples, 1]
+      target        optional, list of numpy arrays [samples, 1]
+      truth_f       optional, numpy array [frames, 1]
+      predict       optional, numpy array [frames, 1]
       label         optional, label name to use when plotting
 
     """
     from sonusai import SonusAIError
     from sonusai.mixture import SAMPLE_RATE
-    from sonusai.utils import int16_to_float
 
     if mixture.ndim != 1:
         raise SonusAIError('Too many dimensions in mixture')
@@ -196,7 +196,7 @@ def class_plot(mixture: np.ndarray,
     fig, ax = plt.subplots(1, 1, constrained_layout=True, figsize=(11, 8.5))
 
     # Plot the time-domain waveforms then truth/prediction on second axis
-    ax.plot(x_axis, int16_to_float(mixture[0:samples]), color='mistyrose', label='Mixture')
+    ax.plot(x_axis, mixture[0:samples], color='mistyrose', label='Mixture')
     color = 'red'
     ax.set_xlim(x_axis[0], x_axis[-1])
     ax.set_ylim([-1.025, 1.025])
@@ -205,7 +205,7 @@ def class_plot(mixture: np.ndarray,
 
     # Plot target time-domain waveform
     if target is not None:
-        ax.plot(x_axis, int16_to_float(target[0:samples]), color='blue', label='Target')
+        ax.plot(x_axis, target[0:samples], color='blue', label='Target')
 
     # instantiate 2nd y-axis that shares the same x-axis
     if truth_f is not None or predict is not None:
@@ -243,46 +243,46 @@ def expand_frames_to_samples(x: np.ndarray, samples: int) -> np.ndarray:
 
 
 def main():
+    from docopt import docopt
+
+    import sonusai
+    from sonusai.utils import trim_docstring
+
+    args = docopt(trim_docstring(__doc__), version=sonusai.__version__, options_first=True)
+
     from os.path import basename
     from os.path import exists
+    from os.path import isdir
     from os.path import splitext
 
     import h5py
-    from docopt import docopt
     from matplotlib.backends.backend_pdf import PdfPages
     from pyaaware import Predict
 
-    import sonusai
     from sonusai import SonusAIError
     from sonusai import create_file_handler
     from sonusai import initial_log_messages
     from sonusai import update_console_handler
-    from sonusai.mixture import DEFAULT_FRAME_SIZE
+    from sonusai.mixture import MixtureDatabase
+    from sonusai.mixture import MixtureDatabaseConfig
     from sonusai.mixture import get_feature_from_audio
-    from sonusai.mixture import get_feature_stats
-    from sonusai.mixture import get_mixture_data
     from sonusai.mixture import get_truth_indices_for_mixid
-    from sonusai.mixture import load_mixdb
     from sonusai.mixture import read_audio
     from sonusai.utils import get_label_names
     from sonusai.utils import print_mixture_details
-    from sonusai.utils import trim_docstring
-
-    args = docopt(trim_docstring(__doc__), version=sonusai.__version__, options_first=True)
 
     verbose = args['--verbose']
     model_name = args['--model']
     output_name = args['--output']
     labels_name = args['--labels']
     mixid = args['--mixid']
-    input_name = args['INPUT']
     energy = args['--energy']
+    input_name = args['INPUT']
 
     if mixid is not None:
         mixid = int(mixid)
 
-    log_name = 'plot.log'
-    create_file_handler(log_name)
+    create_file_handler('plot.log')
     update_console_handler(verbose)
     initial_log_messages('plot')
 
@@ -300,7 +300,6 @@ def main():
     ext = splitext(input_name)[1]
 
     model = None
-    mixdb = None
     target = None
     truth_f = None
     t_indices = None
@@ -313,36 +312,27 @@ def main():
             raise SonusAIError('Must specify MODEL when input is WAV')
 
         mixture = read_audio(input_name)
-        feature = get_feature_from_audio(audio=mixture, model=model)
-        fs = get_feature_stats(feature_mode=model.feature,
-                               frame_size=DEFAULT_FRAME_SIZE,
-                               num_classes=model.output_shape[-1],
-                               truth_mutex=model.mutex)
-        step = fs.step
-        fs.step
-
+        feature = get_feature_from_audio(audio=mixture, feature=model.feature)
+        mixdb = MixtureDatabase(config=MixtureDatabaseConfig(feature=model.feature,
+                                                             num_classes=model.output_shape[-1],
+                                                             truth_mutex=False,
+                                                             truth_reduction_function='max'),
+                                lazy_load=True)
         logger.debug(f'Audio samples      {len(mixture)}')
         logger.debug(f'Feature shape      {feature.shape}')
 
-    elif ext in ['.json', '.h5']:
+    elif isdir(input_name):
         if mixid is None:
-            raise SonusAIError('Must specify MIXID when input is JSON or H5')
+            raise SonusAIError('Must specify mixid when input is mixture database')
 
-        mixdb = load_mixdb(input_name)
+        mixdb = MixtureDatabase(config=input_name, show_progress=True)
 
-        print_mixture_details(mixdb=mixdb, mixid=mixid, desc_len=24, print_fn=logger.info, all_class_counts=False)
+        print_mixture_details(mixdb=mixdb, mixid=mixid, desc_len=24, print_fn=logger.info)
 
         logger.info(f'Generating data for mixture {mixid}')
-        md = get_mixture_data(mixdb=mixdb, mixid=mixid)
-        mixture = md.mixture
-        target = md.target
-        feature = md.feature
-        truth_f = md.truth_f
-        fs = get_feature_stats(feature_mode=mixdb.feature,
-                               frame_size=DEFAULT_FRAME_SIZE,
-                               num_classes=mixdb.num_classes,
-                               truth_mutex=mixdb.truth_mutex)
-        step = fs.step
+        mixture = mixdb.mixture_mixture(mixid)
+        target = mixdb.mixture_target(mixid)
+        feature, truth_f = mixdb.mixture_ft(mixid)
         t_indices = [x - 1 for x in get_truth_indices_for_mixid(mixdb=mixdb, mixid=mixid)]
 
         targets = [mixdb.targets[tfi] for tfi in mixdb.mixtures[mixid].target_file_index]
@@ -368,7 +358,7 @@ def main():
 
     predict = None
     labels = None
-    indices = list()
+    indices = []
     if model is not None:
         logger.debug('')
         logger.info(f'Running prediction on mixture {mixid}')
@@ -411,12 +401,12 @@ def main():
         title = f'{input_name}'
         pdf_name = f'{base_name}-plot.pdf'
 
-    # Original size (frames, stride, num_bands)
+    # Original size [frames, stride, num_bands]
     # Decimate in the stride dimension
     # Reshape to get frames*decimated_stride, num_bands
     if feature.ndim != 3:
         raise SonusAIError(f'feature does not have 3 dimensions: frames, stride, num_bands')
-    spectrogram = feature[:, -step:, :]
+    spectrogram = feature[:, -mixdb.fg.step:, :]
     spectrogram = np.reshape(spectrogram, (spectrogram.shape[0] * spectrogram.shape[1], spectrogram.shape[2]))
 
     with PdfPages(pdf_name) as pdf:
@@ -429,11 +419,11 @@ def main():
             if energy:
                 t_tmp = None
                 if truth_f is not None:
-                    t_tmp = 10*np.log10(truth_f + np.finfo(np.float32).eps)
+                    t_tmp = 10 * np.log10(truth_f + np.finfo(np.float32).eps)
 
                 p_tmp = None
                 if predict is not None:
-                    p_tmp = 10*np.log10(predict + np.finfo(np.float32).eps)
+                    p_tmp = 10 * np.log10(predict + np.finfo(np.float32).eps)
 
                 pdf.savefig(spec_energy_plot(mixture=mixture,
                                              feature=spectrogram,
@@ -456,7 +446,7 @@ def main():
         logger.info(f'Wrote {pdf_name}')
 
     if output_name:
-        with h5py.File(name=output_name, mode='w') as f:
+        with h5py.File(output_name, 'w') as f:
             f.create_dataset(name='predict', data=predict)
             logger.info(f'Wrote {output_name}')
 
