@@ -23,14 +23,11 @@ def _parallel(ordered: bool, function: Callable, *iterables: Iterable, **kwargs:
         A generator which will apply the function to each element of the given Iterables
         in parallel in order.
     """
+    import multiprocessing as mp
     from typing import Sized
 
     from tqdm.auto import tqdm
 
-    from sonusai.utils import initialize_map_func
-
-    map_func = initialize_map_func(ordered, **kwargs)
-
     # Extract progress bar
     progress = kwargs.pop('progress', None)
     progress_needs_close = False
@@ -44,9 +41,25 @@ def _parallel(ordered: bool, function: Callable, *iterables: Iterable, **kwargs:
         progress = tqdm(total=total, **kwargs)
         progress_needs_close = True
 
-    for item in map_func(function, *iterables):
-        yield item
-        progress.update()
+    # Extract num_cpus
+    num_cpus = kwargs.pop('num_cpus', None)
+
+    # Determine num_cpus
+    if num_cpus is None:
+        num_cpus = mp.cpu_count()
+    elif type(num_cpus) == float:
+        num_cpus = int(round(num_cpus * mp.cpu_count()))
+
+    # Create parallel generator
+    initializer = kwargs.pop('initializer', None)
+    initargs = kwargs.pop('initargs', None)
+    map_type = 'imap' if ordered else 'imap_unordered'
+    with mp.Pool(num_cpus, initializer=initializer, initargs=initargs) as pool:
+        map_func = getattr(pool, map_type)
+
+        for item in map_func(function, *iterables):
+            yield item
+            progress.update()
 
     if progress_needs_close:
         progress.close()
