@@ -1,7 +1,9 @@
 import numpy as np
 
+from sonusai.mixture.types import Predict
 
-def read_predict_data(filename: str) -> np.ndarray:
+
+def read_predict_data(filename: str) -> Predict:
     """Read predict data from given HDF5 file and return it."""
     import h5py
 
@@ -9,9 +11,9 @@ def read_predict_data(filename: str) -> np.ndarray:
     from sonusai import logger
 
     logger.debug(f'Reading prediction data from {filename}')
-    with h5py.File(name=filename, mode='r') as f:
+    with h5py.File(filename, 'r') as f:
         # prediction data is either [frames, num_classes], or [frames, timesteps, num_classes]
-        predict = np.array(f['/predict'])
+        predict = np.array(f['predict'])
 
         if predict.ndim == 2:
             return predict
