@@ -14,7 +14,6 @@ def main():
     from docopt import docopt
 
     import sonusai
-    from sonusai.mixture import DEFAULT_FRAME_SIZE
     from sonusai.mixture import DEFAULT_NOISE
     from sonusai.utils import trim_docstring
 
@@ -23,7 +22,6 @@ def main():
     print('Custom SonusAI variables:')
     print('')
     print(f'  ${{default_noise}}: {DEFAULT_NOISE}')
-    print(f'  ${{frame_size}}:    {DEFAULT_FRAME_SIZE}')
     print('')
 
 
